﻿namespace UltraLabelHotkey
{
    using Infragistics.Win;
    using Infragistics.Win.Misc;
    using System.Diagnostics;
    using System.Drawing;
    using System.Linq;
    using System.Windows.Forms;
    using static Infragistics.Win.ImageAndTextUIElement;

    public partial class Form1 : Form
    {
        public Form1()
        {
            Application.AddMessageFilter(new CustomMessageFilter(this));
            InitializeComponent();
            this.ultraLabel1.DrawFilter = new DF(Color.Red, drawUnderline: false);
            this.ultraLabel2.DrawFilter = new DF(Color.Blue, drawUnderline: false);
            this.ultraLabel3.DrawFilter = new DF(Color.Green, drawUnderline: false);
            this.ultraLabel4.DrawFilter = new DF(Color.Magenta, drawUnderline: false);
        }
    }

    internal class CustomMessageFilter : IMessageFilter
    {
        private Form _form = null;

        public CustomMessageFilter(Form form)
        {
            _form = form;
        }

        public bool PreFilterMessage(ref Message m)
        {
            // Handle Alt Key Down and Key Up
            if (m.Msg == 260 || m.Msg == 261)
            {
                Debug.WriteLine(m.Msg == 260 ? "Alt Down" : "Alt Up");
                var ultraLabels = _form.Controls.OfType<UltraLabel>();

                foreach (UltraLabel label in ultraLabels)
                {
                    (label.DrawFilter as DF).DrawUnderline = m.Msg == 260;
                    label.Refresh();
                }

                return true;
            }

            return false;
        }
    }

    internal class DF : IUIElementDrawFilter
    {
        public Color Color { get; set; }
        public bool DrawUnderline { get; set; }

        public DF(Color color, bool drawUnderline)
        {
            DrawUnderline = drawUnderline;
            Color = color;
        }

        public bool DrawElement(DrawPhase drawPhase, ref UIElementDrawParams drawParams)
        {
            var textElement = drawParams.Element as ImageAndTextDependentTextUIElement;
            if(textElement != null)
            {
                //  Get the text of the label
                var text = textElement.Text;

                //  Get the index of the star symbol
                var starIndex = text.IndexOf('*');

                //  Get the index of ampersand symbol
                var focusCharIndex = text.IndexOf('&');

                //  If there is ampersand in the text remove it. Also if the star is after the
                //  ampersand decrease the position of the star as we are making the length of
                //  the text before the star one letter shorter
                if(focusCharIndex > -1)
                {
                    text = text.Substring(0, focusCharIndex) + text.Substring(focusCharIndex + 1);
                    if(starIndex > focusCharIndex)
                    {
                        starIndex--;
                    }
                }

                //  Get the location of the start of the text, the font and the color of the text
                var location = textElement.RectInsideBorders.Location;
                var font = textElement.Control.Font;
                var brush = drawParams.TextBrush;
                var originalColor = ((SolidBrush)drawParams.TextBrush).Color;

                var index = 0;

                //  Draw each letter as necessary by yourself via drawParams' Graphic object
                foreach(var letter in text)
                {
                    //  If we are at the star change the color of the brush to the one you need
                    if(index == starIndex)
                    {
                        ((SolidBrush)brush).Color = Color;
                    }

                    //  If we are at focus char make font Underline
                    if(index == focusCharIndex && DrawUnderline)
                    {
                        font = new Font(font, FontStyle.Underline);
                    }

                    //  Draw current char
                    drawParams.Graphics.DrawString(letter.ToString(), font, brush, location);

                    //  Measure the width of the current char and offset the location accordingly
                    //  to be able to draw the next char at the correct position
                    var letterWidth = drawParams.Graphics.MeasureString(letter.ToString(), font, int.MaxValue, StringFormat.GenericTypographic).Width;
                    location.Offset((int)letterWidth, 0);

                    //  Reset the brush and font
                    ((SolidBrush)brush).Color = originalColor;
                    font = textElement.Control.Font;

                    index++;
                }

                //  Return true so no other painting will be performed
                return true;
            }

            return false;
        }

        public DrawPhase GetPhasesToFilter(ref UIElementDrawParams drawParams)
        {
            //  If the element to be drawn is ImageAndTextDependentTextUIElement return just before
            //  the ForeGround, text, is drawn
            if(drawParams.Element is ImageAndTextDependentTextUIElement)
            {
                return DrawPhase.BeforeDrawForeground;
            }
            return DrawPhase.None;
        }
    }
}
